import 'package:flutter/widgets.dart';

class NavigatorKey<S extends Object> extends GlobalObjectKey<NavigatorState> {
  final String key;

  NavigatorKey(this.key, S value) : super(value) {
    Navigation.navigators[key] = this;
  }

  S get state => value as S;
}

class Navigation {
  static final Map<String, NavigatorKey> navigators = <String, NavigatorKey>{};

  static NavigatorKey<S> keyOf<S extends Object>(String key) {
    return navigators[key] as NavigatorKey<S>;
  }

  static S stateOf<S extends Object>(String key) {
    return navigators[key]?.state as S;
  }

  static BuildContext? contextOf(String key) {
    return navigators[key]?.currentContext;
  }

  static Widget? widgetOf(String key) {
    return navigators[key]?.currentWidget;
  }

  static NavigatorState? of(String key, [BuildContext? context]) {
    return navigators[key]?.currentState ?? (context != null ? Navigator.of(context) : null);
  }
}
